package io.renren.modules.minio.service;

import io.renren.modules.minio.pojo.MinioAlgorithm;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description
 * @Author chenrj.xdu@gmail.com
 * @Version V1.0.0
 * @Since 1.0
 * @Date 2021/10/16
 */

@Service
public interface MinioAlgorithmService {
    /**
     * 分页获取minio系统中的算法
     *
     * @return
     */
    // int offset, int size
    List<MinioAlgorithm> getMinioAlgorithmList();

    int getAlgorithmNum();

    String findModelByName(String algorithmName);

    String findAppFileByName(String algorithmName);

    String findImageFileByName(String algorithmName);

    Long findBelongAlgId(String algorithmName);
}
